//////////////////////////////////////////////////////////////////////////////////
// OpenGL Mathematics Copyright (c) 2005 G-Truc Creation (www.g-truc.net)
//////////////////////////////////////////////////////////////////////////////////
// Author  : Christophe [Groove] Riccio (contact [at] g-truc [point] net)
// Created : 2005-01-16
// Updated : 2006-01-04
// Licence : This source is under GNU LGPL licence
// File    : _mat2.inl
//////////////////////////////////////////////////////////////////////////////////

#ifndef __mat2_inl__
#define __mat2_inl__

#include "./_mat2.h"

namespace glm
{
    //////////////////////////////////////////////////////////////
    // mat2 constructors

    template <typename T> 
    inline _xmat2<T>::_xmat2()
    {
        this->value[0] = _xvec2<T>(1, 0);
        this->value[1] = _xvec2<T>(0, 1);
    }

    template <typename T> 
    inline _xmat2<T>::_xmat2(const T f)
    {
        this->value[0] = _xvec2<T>(f, 0);
        this->value[1] = _xvec2<T>(0, f);
    }

    template <typename T> 
    inline _xmat2<T>::_xmat2(const T x0, const T y0, const T x1, const T y1)
    {
        this->value[0] = _xvec2<T>(x0, y0);
        this->value[1] = _xvec2<T>(x1, y1);
    }

    template <typename T> 
    inline _xmat2<T>::_xmat2(const _xvec2<T> & v0, const _xvec2<T> & v1)
    {
        this->value[0] = v0;
        this->value[1] = v1;
    }
/*
    template <typename T> 
    inline _xmat2<T>::_xmat2(const T* a)
    {
        this->value[0] = _xvec2<T>(a[0], a[1]);
        this->value[1] = _xvec2<T>(a[2], a[3]);
    }
*/
    //////////////////////////////////////////////////////////////
    // mat2 operators

    template <typename T> 
    inline _xmat2<T> & _xmat2<T>::operator=(const _xmat2<T> & m)
    {
	    this->value[0] = m[0];
	    this->value[1] = m[1];
        return *this;
    }

    template <typename T> 
    inline _xmat2<T> _xmat2<T>::operator+ (const _xmat2<T> & m) const
    {
	    return _xmat2<T>(
		    this->value[0] + m[0],
		    this->value[1] + m[1]);
    }

    template <typename T> 
    inline _xmat2<T> & _xmat2<T>::operator+= (const _xmat2<T> & m)
    {
	    this->value[0] += m[0];
	    this->value[1] += m[1];
	    return *this;
    }

    template <typename T> 
    inline _xmat2<T> _xmat2<T>::operator- (const _xmat2<T> & m) const
    {
	    return _xmat2<T>(
		    this->value[0] - m[0],
		    this->value[1] - m[1]);
    }

    template <typename T> 
    inline _xmat2<T> & _xmat2<T>::operator-= (const _xmat2<T> & m)
    {
	    this->value[0] -= m[0];
	    this->value[1] -= m[1];
	    return *this;
    }

    template <typename T> 
    inline _xmat2<T> _xmat2<T>::operator* (const T s) const
    {
	    return _xmat2<T> (
		    this->value[0] * s,
		    this->value[1] * s);
    }

    template <typename T> 
    inline _xmat2<T> & _xmat2<T>::operator*= (const T s)
    {
	    this->value[0] *= s;
	    this->value[1] *= s;
	    return *this;
    }

    template <typename T> 
    inline _xvec2<T> _xmat2<T>::operator* (const _xvec2<T> & v) const
    {
        return _xvec2<T>(
            this->value[0][0] * v.x + this->value[1][0] * v.y,
            this->value[0][1] * v.x + this->value[1][1] * v.y);
    }

    template <typename T> 
    inline _xmat2<T> _xmat2<T>::operator/ (const T s) const
    {
	    return _xmat2<T>(
		    this->value[0] / s,
	        this->value[1] / s);
    }

    template <typename T> 
    inline _xmat2<T> & _xmat2<T>::operator/= (const T s)
    {
	    this->value[0] /= s;
	    this->value[1] /= s;
	    return *this;
    }

    template <typename T> 
    inline _xmat2<T> _xmat2<T>::operator* (const _xmat2<T> & m) const
    {
        return _xmat2<T>(
            this->value[0][0] * m[0][0] + this->value[1][0] * m[0][1],
            this->value[0][1] * m[0][0] + this->value[1][1] * m[0][1],
            this->value[0][1] * m[1][0] + this->value[1][1] * m[1][1],
            this->value[0][0] * m[1][0] + this->value[1][0] * m[1][1]);
    }

    template <typename T> 
    inline _xmat2<T>& _xmat2<T>::operator*= (const _xmat2<T> & m)
    {
        return *this = *this * m;
    }

    template <typename T> 
    inline _xmat2<T> _xmat2<T>::operator/ (const _xmat2<T> & m) const
    {
        T Determinant = m[0][0] * m[1][1] - m[1][0] * m[0][1];

        _xmat2<T> Inverse(
            + m[1][1] / Determinant,
            - m[1][0] / Determinant,
            - m[0][1] / Determinant, 
            + m[0][0] / Determinant);

        return *this * Inverse;
    }

    template <typename T>
    inline _xmat2<T>& _xmat2<T>::operator/= (const _xmat2<T> & m)
    {
        return *this = *this / m;
    }

    template <typename T> 
    inline _xmat2<T> _xmat2<T>::operator- () const
    {
	    return _xmat2<T>(
            -this->value[0], 
            -this->value[1]);
    }

    template <typename T> 
    inline _xmat2<T> _xmat2<T>::operator-- ()
    {
	    this->value[0]--;
	    this->value[1]--;
	    return *this;
    }

    template <typename T>
    inline const _xmat2<T> _xmat2<T>::operator-- (int n) const 
    {
        _xmat2<T> m = *this;
	    m.value[0]--;
	    m.value[1]--;
	    return m;
    }

    template <typename T> 
    inline _xmat2<T> _xmat2<T>::operator++ ()
    {
	    this->value[0]++;
	    this->value[1]++;
	    return *this;
    }

    template <typename T>
    inline const _xmat2<T> _xmat2<T>::operator++ (int n) const
    {
        _xmat2<T> m = *this;
	    m.value[0]++;
	    m.value[1]++;
	    return m;
    }
} // namespace glm

#endif //__mat2_inl__
