//////////////////////////////////////////////////////////////////////////////////
// OpenGL Mathematics Copyright (c) 2005 G-Truc Creation (www.g-truc.net)
//////////////////////////////////////////////////////////////////////////////////
// Author  : Christophe [Groove] Riccio (contact [at] g-truc [point] net)
// Created : 2005-01-27
// Updated : 2006-01-04
// Licence : This source is under GNU LGPL licence
// File    : _mat3.inl
//////////////////////////////////////////////////////////////////////////////////

#ifndef __mat3_inl__
#define __mat3_inl__

#include "./_mat3.h"
#include "./_func.h"

namespace glm
{
    //////////////////////////////////////////////////////////////
    // mat3 constructors

    template <typename T> 
    inline _xmat3<T>::_xmat3()
    {
        this->value[0] = _xvec3<T>(1, 0, 0);
        this->value[1] = _xvec3<T>(0, 1, 0);
        this->value[2] = _xvec3<T>(0, 0, 1);
    }

    template <typename T> 
    inline _xmat3<T>::_xmat3(const T f)
    {
        this->value[0] = _xvec3<T>(f, 0, 0);
        this->value[1] = _xvec3<T>(0, f, 0);
        this->value[2] = _xvec3<T>(0, 0, f);
    }

    template <typename T> 
    inline _xmat3<T>::_xmat3
    (
        const T x0, const T y0, const T z0, 
        const T x1, const T y1, const T z1,
        const T x2, const T y2, const T z2
    )
    {
        this->value[0] = _xvec3<T>(x0, y0, z0);
        this->value[1] = _xvec3<T>(x1, y1, z1);
        this->value[2] = _xvec3<T>(x2, y2, z2);
    }

    template <typename T> 
    inline _xmat3<T>::_xmat3
    (
        const _xvec3<T> & v0, 
        const _xvec3<T> & v1, 
        const _xvec3<T> & v2
    )
    {
        this->value[0] = v0;
        this->value[1] = v1;
        this->value[2] = v2;
    }
/*
    template <typename T> 
    inline _xmat3<T>::_xmat3(const T* a)
    {
        this->value[0] = _xvec3<T>(a[0], a[1], a[2]);
        this->value[1] = _xvec3<T>(a[3], a[4], a[5]);
        this->value[2] = _xvec3<T>(a[6], a[7], a[8]);
    }
*/
    /*
    // GL_GTX_euler_angles
    template <typename T> 
    inline _xmat3<T>::_xmat3(const _xvec3<T> & angles)
    {
        T ch = cos(angles.x);
        T sh = sin(angles.x);
        T cp = cos(angles.y);
        T sp = sin(angles.y);
        T cb = cos(angles.z);
        T sb = sin(angles.z);

        value[0][0] = ch * cb + sh * sp * sb;
        value[0][1] = sb * cp;
        value[0][2] = -sh * cb + ch * sp * sb;
        value[1][0] = -ch * sb + sh * sp * cb;
        value[1][1] = cb * cp;
        value[1][2] = sb * sh + ch * sp * cb;
        value[2][0] = sh * cp;
        value[2][1] = -sp;
        value[2][2] = ch * cp;
    }
    */
    //////////////////////////////////////////////////////////////
    // mat3 conversions
    /*
    template <typename T> 
    inline _xmat3<T>::_xmat3(const _xquat<T> & q)
    {
        this->value[0][0] = 1 - 2 * q.y * q.y - 2 * q.z * q.z;
        this->value[0][1] = 2 * q.x * q.y + 2 * q.w * q.z;
        this->value[0][2] = 2 * q.x * q.z - 2 * q.w * q.y;

        this->value[1][0] = 2 * q.x * q.y - 2 * q.w * q.z;
        this->value[1][1] = 1 - 2 * q.x * q.x - 2 * q.z * q.z;
        this->value[1][2] = 2 * q.y * q.z + 2 * q.w * q.x;

        this->value[2][0] = 2 * q.x * q.z + 2 * q.w * q.y;
        this->value[2][1] = 2 * q.y * q.z - 2 * q.w * q.x;
        this->value[2][2] = 1 - 2 * q.x * q.x - 2 * q.y * q.y;
    }
    */
    //////////////////////////////////////////////////////////////
    // mat3 operators

    template <typename T> 
    inline _xmat3<T> & _xmat3<T>::operator=(const _xmat3<T> & m)
    {
	    this->value[0] = m[0];
	    this->value[1] = m[1];
        this->value[2] = m[2];
	    return *this;
    }

    template <typename T> 
    inline _xmat3<T> _xmat3<T>::operator+ (const _xmat3<T> & m) const
    {
	    return _xmat3<T>(
		    this->value[0] + m[0],
		    this->value[1] + m[1],
            this->value[2] + m[2]);
    }

    template <typename T> 
    inline _xmat3<T> & _xmat3<T>::operator+= (const _xmat3<T> & m)
    {
	    this->value[0] += m[0];
	    this->value[1] += m[1];
        this->value[2] += m[2];
	    return *this;
    }

    template <typename T> 
    inline _xmat3<T> _xmat3<T>::operator- (const _xmat3<T> & m) const
    {
	    return _xmat3<T>(
		    this->value[0] - m[0],
		    this->value[1] - m[1],
            this->value[2] - m[2]);
    }

    template <typename T> 
    inline _xmat3<T> & _xmat3<T>::operator-= (const _xmat3<T> & m)
    {
	    this->value[0] -= m[0];
	    this->value[1] -= m[1];
        this->value[2] -= m[2];
	    return *this;
    }

    template <typename T> 
    inline _xmat3<T> _xmat3<T>::operator* (const T s) const
    {
	    return _xmat3<T>(
		    this->value[0] * s,
		    this->value[1] * s,
            this->value[2] * s);
    }

    template <typename T> 
    inline _xmat3<T> & _xmat3<T>::operator*= (const T s)
    {
	    this->value[0] *= s;
	    this->value[1] *= s;
        this->value[2] *= s;
	    return *this;
    }

    template <typename T> 
    inline _xvec3<T> _xmat3<T>::operator* (const _xvec3<T> & v) const
    {
        return _xvec3<T>(
            this->value[0][0] * v.x + this->value[1][0] * v.y + this->value[2][0] * v.z,
            this->value[0][1] * v.x + this->value[1][1] * v.y + this->value[2][1] * v.z,
            this->value[0][2] * v.x + this->value[1][2] * v.y + this->value[2][2] * v.z);
    }

    template <typename T> 
    inline _xmat3<T> _xmat3<T>::operator/ (const T s) const
    {
	    return _xmat3<T>(
		    this->value[0] / s,
	        this->value[1] / s,
            this->value[2] / s);
    }

    template <typename T> 
    inline _xmat3<T> & _xmat3<T>::operator/= (const T s)
    {
	    this->value[0] /= s;
	    this->value[1] /= s;
        this->value[2] /= s;
	    return *this;
    }

    template <typename T> 
    inline _xmat3<T> _xmat3<T>::operator* (const _xmat3<T> & m) const
    {
        return _xmat3<T>(
            this->value[0][0] * m[0][0] + this->value[1][0] * m[0][1] + this->value[2][0] * m[0][2],
            this->value[0][1] * m[0][0] + this->value[1][1] * m[0][1] + this->value[2][1] * m[0][2],
            this->value[0][2] * m[0][0] + this->value[1][2] * m[0][1] + this->value[2][2] * m[0][2],

            this->value[0][0] * m[1][0] + this->value[1][0] * m[1][1] + this->value[2][0] * m[1][2],
            this->value[0][1] * m[1][0] + this->value[1][1] * m[1][1] + this->value[2][1] * m[1][2],
            this->value[0][2] * m[1][0] + this->value[1][2] * m[1][1] + this->value[2][2] * m[1][2],

            this->value[0][0] * m[2][0] + this->value[1][0] * m[2][1] + this->value[2][0] * m[2][2],
            this->value[0][1] * m[2][0] + this->value[1][1] * m[2][1] + this->value[2][1] * m[2][2],
            this->value[0][2] * m[2][0] + this->value[1][2] * m[2][1] + this->value[2][2] * m[2][2]);
    }

    template <typename T> 
    inline _xmat3<T> & _xmat3<T>::operator*= (const _xmat3<T> & m)
    {
        return (*this = *this * m);
    }

    template <typename T> 
    inline _xmat3<T> _xmat3<T>::operator/ (const _xmat3<T> & m) const
    {
        T Determinant = m[0][0] * (m[1][1] * m[2][2] - m[2][1] * m[1][2])
                    - m[1][0] * (m[0][1] * m[2][2] - m[2][1] * m[0][2])
                    + m[2][0] * (m[0][1] * m[1][2] - m[1][1] * m[0][2]);

        _xmat3<T> Inverse(
            + (m[1][1] * m[2][2] - m[2][1] * m[1][2]),
            - (m[1][0] * m[2][2] - m[2][0] * m[1][2]),
            + (m[1][0] * m[2][1] - m[2][0] * m[1][1]),
            - (m[0][1] * m[2][2] - m[2][1] * m[0][2]),
            + (m[0][0] * m[2][2] - m[2][0] * m[0][2]),
            - (m[0][0] * m[2][1] - m[2][0] * m[0][1]),
            + (m[0][1] * m[1][2] - m[1][1] * m[0][2]),
            - (m[0][0] * m[1][2] - m[1][0] * m[0][2]),
            + (m[0][0] * m[1][1] - m[1][0] * m[0][1]));
        Inverse /= Determinant;

        return *this * Inverse;
    }

    template <typename T> 
    inline _xmat3<T> & _xmat3<T>::operator/= (const _xmat3<T> & m)
    {
        return (*this = *this / m);
    }

    template <typename T> 
    inline _xmat3<T> _xmat3<T>::operator- () const
    {
	    return _xmat3<T>(
            -this->value[0], 
            -this->value[1],
            -this->value[2]);
    }

    template <typename T> 
    inline _xmat3<T> _xmat3<T>::operator-- ()
    {
	    this->value[0]--;
	    this->value[1]--;
        this->value[2]--;
	    return *this;
    }

    template <typename T> 
    inline const _xmat3<T> _xmat3<T>::operator-- (int n) const 
    {
        _xmat3<T> m = *this;
	    m.value[0]--;
	    m.value[1]--;
        m.value[2]--;
	    return m;
    }

    template <typename T> 
    inline _xmat3<T> _xmat3<T>::operator++ ()
    {
	    this->value[0]++;
	    this->value[1]++;
        this->value[2]++;
	    return *this;
    }

    template <typename T> 
    inline const _xmat3<T> _xmat3<T>::operator++ (int n) const
    {
        _xmat3<T> m = *this;
	    m.value[0]++;
	    m.value[1]++;
        m.value[2]++;
	    return m;
    }
} // namespace glm

#endif //__mat3_inl__
