//////////////////////////////////////////////////////////////////////////////////
// OpenGL Mathematics Copyright (c) 2005 G-Truc Creation (www.g-truc.net)
//////////////////////////////////////////////////////////////////////////////////
// Author  : Christophe [Groove] Riccio (contact [at] g-truc [point] net)
// Created : 2005-12-21
// Updated : 2005-12-21
// Licence : This source is under GNU LGPL licence
// File    : gtx_euler_angles.inl
//////////////////////////////////////////////////////////////////////////////////
// Dependency:
// - GL_GTX_double
//////////////////////////////////////////////////////////////////////////////////
// ToDo:
// - mat2 mat2GTX(const vec2& angles) undefined
// - mat3 mat3GTX(const vec2& angles) undefined
// - Add functions to double numbers
//////////////////////////////////////////////////////////////////////////////////

#ifndef __gtx_euler_angles_inl__
#define __gtx_euler_angles_inl__

#include "gtx_euler_angles.h"

namespace glm
{
    template <typename T> 
	inline _xmat2<T> mat2GTX(const T angle)
	{
        T c = glm::cos(angle);
        T s = glm::sin(angle);

		_xmat3<T> Result;
        Result[0][0] = c;
        Result[0][1] = s;
        Result[1][0] = -s;
        Result[1][1] = c;
		return Result;
	}

    template <typename T> 
	inline _xmat3<T> mat3GTX(const T angle)
	{
        T c = glm::cos(angle);
        T s = glm::sin(angle);

		_xmat3<T> Result;
        Result[0][0] = c;
        Result[0][1] = s;
		Result[0][2] = 0.0f;
        Result[1][0] = -s;
        Result[1][1] = c;
		Result[1][2] = 0.0f;
		Result[2][0] = 0.0f;
		Result[2][1] = 0.0f;
		Result[2][2] = 1.0f;
		return Result;
	}

	template <typename T>
    inline _xmat3<T> mat3GTX(const _xvec3<T>& angles)
    {
        T ch = glm::cos(angles.x);
        T sh = glm::sin(angles.x);
        T cp = glm::cos(angles.y);
        T sp = glm::sin(angles.y);
        T cb = glm::cos(angles.z);
        T sb = glm::sin(angles.z);

        _xmat3<T> Result;
        Result[0][0] = ch * cb + sh * sp * sb;
        Result[0][1] = sb * cp;
        Result[0][2] = -sh * cb + ch * sp * sb;
        Result[1][0] = -ch * sb + sh * sp * cb;
        Result[1][1] = cb * cp;
        Result[1][2] = sb * sh + ch * sp * cb;
        Result[2][0] = sh * cp;
        Result[2][1] = -sp;
        Result[2][2] = ch * cp;
        return Result;
    }

	template <typename T>
    inline _xmat4<T> mat4GTX(const _xvec3<T>& angles)
    {
        T ch = glm::cos(angles.x);
        T sh = glm::sin(angles.x);
        T cp = glm::cos(angles.y);
        T sp = glm::sin(angles.y);
        T cb = glm::cos(angles.z);
        T sb = glm::sin(angles.z);

        _xmat4<T> Result;
        Result[0][0] = ch * cb + sh * sp * sb;
        Result[0][1] = sb * cp;
        Result[0][2] = -sh * cb + ch * sp * sb;
        Result[0][3] = 0.0f;
        Result[1][0] = -ch * sb + sh * sp * cb;
        Result[1][1] = cb * cp;
        Result[1][2] = sb * sh + ch * sp * cb;
        Result[1][3] = 0.0f;
        Result[2][0] = sh * cp;
        Result[2][1] = -sp;
        Result[2][2] = ch * cp;
        Result[2][3] = 0.0f;
        Result[3][0] = 0.0f;
        Result[3][1] = 0.0f;
        Result[3][2] = 0.0f;
        Result[3][3] = 1.0f;
        return Result;
    }
}

#endif//__gtx_euler_angles_inl__
