//////////////////////////////////////////////////////////////////////////////////
// OpenGL Mathematics Copyright (c) 2005 G-Truc Creation (www.g-truc.net)
//////////////////////////////////////////////////////////////////////////////////
// Author  : Christophe [Groove] Riccio (contact [at] g-truc [point] net)
// Created : 2005-12-21
// Updated : 2005-12-21
// Licence : This source is under GNU LGPL licence
// File    : gtx_quaternion.inl
//////////////////////////////////////////////////////////////////////////////////

#ifndef __gtx_quaternion_inl__
#define __gtx_quaternion_inl__

#include "gtx_quaternion.h"

namespace glm
{
    template <typename T> 
    inline _xquat<T>::_xquat() : 
        w(1),
        x(0),
        y(0),
        z(0)
    {}

    template <typename T> 
    inline _xquat<T>::_xquat(const T s, const _xvec3<T>& v) : 
        w(s),
        x(v.x),
        y(v.y),
        z(v.z)
    {}

    template <typename T> 
    inline _xquat<T>::_xquat(const T w, const T x, const T y, const T z) :
        w(w),
        x(x),
        y(y),
        z(z)
    {}

    //////////////////////////////////////////////////////////////
    // __quatGT conversions

	template <typename T> 
	inline _xquat<T>::_xquat(const T pitch, const T yaw, const T roll)
	{
		_xvec3<T> eulerAngle(pitch * 0.5f, yaw * 0.5f, roll * 0.5f);
		_xvec3<T> c = glm::cos(eulerAngle * 0.5f);
		_xvec3<T> s = glm::sin(eulerAngle * 0.5f);
		
		this->w = c.x * c.y * c.z + s.x * s.y * s.z;
		this->x = s.x * c.y * c.z - c.x * s.y * s.z;
		this->y = c.x * s.y * c.z + s.x * c.y * s.z;
		this->z = c.x * c.y * s.z - s.x * s.y * c.z;
	}

	template <typename T> 
	inline _xquat<T>::_xquat(const _xvec3<T>& eulerAngle)
	{
		_xvec3<T> c = glm::cos(eulerAngle * 0.5f);
		_xvec3<T> s = glm::sin(eulerAngle * 0.5f);
		
		this->w = c.x * c.y * c.z + s.x * s.y * s.z;
		this->x = s.x * c.y * c.z - c.x * s.y * s.z;
		this->y = c.x * s.y * c.z + s.x * c.y * s.z;
		this->z = c.x * c.y * s.z - s.x * s.y * c.z;		
	}

    /*
    template <typename T> 
    inline glm::_xquat<T>::_xquat(const _xvec4<T>& x) :
        w(x.w),
        x(x.x),
        y(x.y),
        z(x.z)
    {}
    */

    template <typename T> 
    inline _xquat<T>::_xquat(const _xmat3<T>& m)
    {
        T fourXSquaredMinus1 = m[0][0] - m[1][1] - m[2][2];
        T fourYSquaredMinus1 = m[1][1] - m[0][0] - m[2][2];
        T fourZSquaredMinus1 = m[2][2] - m[0][0] - m[1][1];
        T fourWSquaredMinus1 = m[0][0] + m[1][1] + m[2][2];
        
        int biggestIndex = 0;
        T fourBiggestSquaredMinus1 = fourWSquaredMinus1;
        if(fourXSquaredMinus1 > fourBiggestSquaredMinus1)
        {
            fourBiggestSquaredMinus1 = fourXSquaredMinus1;
            biggestIndex = 1;
        }
        if(fourYSquaredMinus1 > fourBiggestSquaredMinus1)
        {
            fourBiggestSquaredMinus1 = fourYSquaredMinus1;
            biggestIndex = 2;
        }
        if(fourZSquaredMinus1 > fourBiggestSquaredMinus1)
        {
            fourBiggestSquaredMinus1 = fourZSquaredMinus1;
            biggestIndex = 3;
        }

        T biggestVal = glm::sqrt(fourBiggestSquaredMinus1 + 1) * 0.5f;
        T mult = 0.25f / biggestVal;

        switch(biggestIndex)
        {
        case 0:
            this->x = (m[1][2] - m[2][1]) * mult;
            this->y = (m[2][0] - m[0][2]) * mult;
            this->z = (m[0][1] - m[1][0]) * mult;
            this->w = biggestVal;
            break;
        case 1:
            this->x = biggestVal;
            this->y = (m[0][1] + m[1][0]) * mult;
            this->z = (m[2][1] + m[1][2]) * mult;
            this->w = (m[1][2] - m[2][1]) * mult;
            break;
        case 2:
            this->x = (m[0][1] + m[1][0]) * mult;
            this->y = biggestVal;
            this->z = (m[1][2] + m[2][1]) * mult;
            this->w = (m[2][0] - m[0][2]) * mult;
            break;
        case 3:
            this->x = (m[2][0] + m[0][2]) * mult;
            this->y = (m[1][2] + m[2][1]) * mult;
            this->z = biggestVal;
            this->w = (m[0][1] - m[1][0]) * mult;
            break;
        }
    }

    template <typename T> 
    inline _xquat<T>::_xquat(const _xmat4<T>& m)
    {
        T fourXSquaredMinus1 = m[0][0] - m[1][1] - m[2][2];
        T fourYSquaredMinus1 = m[1][1] - m[0][0] - m[2][2];
        T fourZSquaredMinus1 = m[2][2] - m[0][0] - m[1][1];
        T fourWSquaredMinus1 = m[0][0] + m[1][1] + m[2][2];
        
        int biggestIndex = 0;
        T fourBiggestSquaredMinus1 = fourWSquaredMinus1;
        if(fourXSquaredMinus1 > fourBiggestSquaredMinus1)
        {
            fourBiggestSquaredMinus1 = fourXSquaredMinus1;
            biggestIndex = 1;
        }
        if(fourYSquaredMinus1 > fourBiggestSquaredMinus1)
        {
            fourBiggestSquaredMinus1 = fourYSquaredMinus1;
            biggestIndex = 2;
        }
        if(fourZSquaredMinus1 > fourBiggestSquaredMinus1)
        {
            fourBiggestSquaredMinus1 = fourZSquaredMinus1;
            biggestIndex = 3;
        }

        T biggestVal = glm::sqrt(fourBiggestSquaredMinus1 + 1) * 0.5f;
        T mult = 0.25f / biggestVal;

        switch(biggestIndex)
        {
        case 0:
            this->x = (m[1][2] - m[2][1]) * mult;
            this->y = (m[2][0] - m[0][2]) * mult;
            this->z = (m[0][1] - m[1][0]) * mult;
            this->w = biggestVal;
            break;
        case 1:
            this->x = biggestVal;
            this->y = (m[0][1] + m[1][0]) * mult;
            this->z = (m[2][1] + m[1][2]) * mult;
            this->w = (m[1][2] - m[2][1]) * mult;
            break;
        case 2:
            this->x = (m[0][1] + m[1][0]) * mult;
            this->y = biggestVal;
            this->z = (m[1][2] + m[2][1]) * mult;
            this->w = (m[2][0] - m[0][2]) * mult;
            break;
        case 3:
            this->x = (m[2][0] + m[0][2]) * mult;
            this->y = (m[1][2] + m[2][1]) * mult;
            this->z = biggestVal;
            this->w = (m[0][1] - m[1][0]) * mult;
            break;
        }
    }
    /*
    template <typename T> 
    inline _xquat<T>::_xquat(float angle, float x, float y, float z)
    {
        // Axis of rotation must be normalised
        float len = glm::length(glm::_xvec3<T>(x, y, z));
        if(glm::abs(len - 1.0f) > 0.001f)
        {
            float oneOverLen = 1 / len;
            x *= oneOverLen;
            y *= oneOverLen;
            z *= oneOverLen;
        }

        angle = glm::radians(angle);
        float fSin = glm::sin(angle * 0.5f);

        this->x = x * fSin;
        this->y = y * fSin;
        this->z = z * fSin;
        this->w = glm::cos(angle * 0.5f));
    }

    template <typename T> 
    inline _xquat<T>::_xquat(float angle, const glm::_xvec3<T>& v)
    {
        glm::_xvec3<T> v_normalized = glm::normalize(v);

        angle = glm::radians(angle);
        float fSin = glm::sin(angle * 0.5f);

        this->x = v.x * fSin;
        this->y = v.y * fSin;
        this->z = v.z * fSin;
        this->w = glm::cos(angle * 0.5f));
    }
    */
    //////////////////////////////////////////////////////////////
    // __quatGT accesses

    template <typename T> 
    inline T& _xquat<T>::operator [] (int i)
    {
        return (&x)[i];
    }

    template <typename T> 
    inline T _xquat<T>::operator [] (int i) const
    {
        return (&x)[i];
    }

    template <typename T> 
    inline _xquat<T>::operator T* ()
    {
        return &x;
    }

    template <typename T> 
    inline _xquat<T>::operator const T* () const 
    {
        return &x;
    }

    //////////////////////////////////////////////////////////////
    // __quatGT Operators

    template <typename T> 
    inline _xquat<T> _xquat<T>::operator +(const _xquat<T>& q) const
    {
        return glm::_xquat<T>(
            this->w + q.w,
            this->x + q.x,
            this->y + q.y,
            this->z + q.z);
    }

    template <typename T> 
    inline _xquat<T>& _xquat<T>::operator +=(const _xquat<T>& q)
    {
        return *this = *this + q;
    }

    template <typename T> 
    inline _xquat<T> _xquat<T>::operator -(const _xquat<T>& q) const
    {
        return _xquat<T>(
            this->w - q.w,
            this->x - q.x,
            this->y - q.y,
            this->z - q.z);
    }

    template <typename T> 
    inline _xquat<T>& _xquat<T>::operator -=(const _xquat<T>& q)
    {
        return *this = *this - q;
    }

    template <typename T> 
    inline _xquat<T> _xquat<T>::operator *(T s) const
    {
        return _xquat<T>(
            this->w * s,
            this->x * s, 
            this->y * s,
            this->z * s);
    }

    template <typename T> 
    inline _xquat<T>& _xquat<T>::operator *=(T s)
    {
        this->w *= s;
        this->x *= s;
        this->y *= s;
        this->z *= s;
        return *this;
    }

    template <typename T>
    inline _xquat<T> _xquat<T>::operator* (const _xquat<T>& q) const
    {
        return _xquat<T>(this->w * q.w, this->x * q.x, this->y * q.y, this->z * q.z);
    }

    template <typename T> 
    inline _xquat<T>& _xquat<T>::operator *=(const _xquat<T>& q)
    {
        return *this = *this * q;
    }

    template <typename T> 
    inline _xquat<T> _xquat<T>::operator /(T s) const
    {
        return glm::_xquat<T>(
            this->w / s,
            this->x / s, 
            this->y / s, 
            this->z / s);
    }

    template <typename T> 
    inline _xquat<T>& _xquat<T>::operator /=(T s)
    {
        this->w /= s;
        this->x /= s;
        this->y /= s;
        this->z /= s;
        return *this;
    }

    template <typename T> 
    inline _xquat<T> _xquat<T>::operator /(const _xquat<T>& q) const
    {
        return *this * inverseGTX(q);
    }

    template <typename T> 
    inline _xquat<T>& _xquat<T>::operator /=(const _xquat<T>& q)
    {
        return *this = *this * glm::inverseGTX(q);
    }

    template <typename T>
    inline _xquat<T> _xquat<T>::operator- () const
    {
        return _xquat<T>(-this->w, -this->x, -this->y, -this->z);
    }

    // Transformation
    template <typename T>
    inline _xvec3<T> _xquat<T>::operator* (const _xvec3<T>& v) const
    {
        _xvec3<T> uv, uuv;
	    _xvec3<T> QuatVector(x, y, z);
        uv = cross(QuatVector, v);
        uuv = cross(QuatVector, uv);
	    uv *= (2.0f * w); 
	    uuv *= 2.0f; 

	    return v + uv + uuv;
    }

    template <typename T>
    inline _xvec4<T> _xquat<T>::operator* (const _xvec4<T>& v) const
    {
        _xvec3<T> uv, uuv;
	    _xvec3<T> QuatVector(x, y, z);
        uv = glm::cross(QuatVector, _xvec3<T>(v));
        uuv = glm::cross(QuatVector, uv);
	    uv *= (2.0f * w); 
	    uuv *= 2.0f; 

	    return _xvec4<T>(v + uv + uuv, v.w);
    }

    ////////////////////////////////////////////////////////
    template <typename T> 
    inline float length(const _xquat<T> & q)
    {
        return static_cast<T>(glm::sqrt(
            q.x * q.x + 
            q.y * q.y + 
            q.z * q.z + 
            q.w * q.w));
    }

    template <typename T> 
    inline _xquat<T> normalize(const _xquat<T>& q)
    {
        T len = static_cast<T>(glm::length(q));
        if(len <= 0) // Problem
            return glm::_xquat<T>(1, 0, 0, 0);
        T oneOverLen = 1 / len;
        return glm::_xquat<T>(q.w * oneOverLen, q.x * oneOverLen, q.y * oneOverLen, q.z * oneOverLen);
    }

    template <typename T> 
    inline T dot(const glm::_xquat<T> & q1, const glm::_xquat<T> & q2)
    {
        return q1.x * q2.x + q1.y * q2.y + q1.z * q2.z + q1.w * q2.w;
    }

    template <typename T> 
    inline _xquat<T> cross(const _xquat<T>& q1, const _xquat<T>& q2)
    {
        return _xquat<T>(
            q1.w * q2.w - q1.x * q2.x - q1.y * q2.y - q1.z * q2.z,
	        q1.w * q2.x + q1.x * q2.w + q1.y * q2.z - q1.z * q2.y,
	        q1.w * q2.y + q1.y * q2.w + q1.z * q2.x - q1.x * q2.z,
	        q1.w * q2.z + q1.z * q2.w + q1.x * q2.y - q1.y * q2.x);
    }
    /*
    template <typename T> 
    inline glm::_xquat<T> glm::cross(const glm::_xquat<T> & q, const glm::_xvec3<T> & v)
    {
        return glm::_xquat<T>(
            (q.w * v.x) + (q.y * v.z) - (q.z * v.y),
            (q.w * v.y) + (q.z * v.x) - (q.x * v.z),
            (q.w * v.z) + (q.x * v.y) - (q.y * v.x),
        -(q.x * v.x) - (q.y * v.y) - (q.z * v.z));
    }
    */
    /*
    template <typename T> 
    inline glm::_xquat<T> glm::cross(const glm::_xvec3<T> & v, const glm::_xquat<T> & q)
    {

    }
    */
    template <typename T>
    inline _xquat<T> mix(const _xquat<T>& x, const _xquat<T>& y, float a)
    {
        if(a <= 0) return x;
        if(a >= 1) return y;

        float fCos = glm::dot(x, y);
        glm::_xquat<T> y2(y); //BUG!!! glm::_xquat<T> y2;
        if(fCos < 0.0f)
        {
            y2 = -y;
            fCos = -fCos;
        }

        //if(fCos > 1.0f) // problem
        float k0, k1;
        if(fCos > 0.9999f)
        {
            k0 = 1.0f - a;
            k1 = 0.0f + a; //BUG!!! 1.0f + a;
        }
        else
        {
            float fSin = glm::sqrt(1.0f - fCos * fCos);
            float fAngle = glm::atan(fSin, fCos);
            float fOneOverSin = 1.0f / fSin;
            k0 = glm::sin((1.0f - a) * fAngle) * fOneOverSin;
            k1 = glm::sin((0.0f + a) * fAngle) * fOneOverSin;
        }

        return glm::_xquat<T>(
            k0 * x.w + k1 * y2.w,
            k0 * x.x + k1 * y2.x,
            k0 * x.y + k1 * y2.y,
            k0 * x.z + k1 * y2.z);
    }

    template <typename T> 
    inline _xquat<T> pow(const _xquat<T>& x, T y)
    {
        if(glm::abs(x.w) > 0.9999f)
            return x;
        float Angle = glm::acos(y);
        float NewAngle = Angle * y;
        float Div = glm::sin(NewAngle) / glm::sin(Angle);
        return glm::_xquat<T>(
            glm::cos(NewAngle),
            x.x * Div,
            x.y * Div,
            x.z * Div);
    }

    template <typename T> 
    inline _xquat<T> conjugateGTX(const _xquat<T>& q)
    {
        return _xquat<T>(q.w, -q.x, -q.y, -q.z);
    }

    template <typename T> 
    inline _xquat<T> inverseGTX(const _xquat<T>& q)
    {
        return conjugateGTX(q) / length(q);
    }

    template <typename T> 
    inline _xquat<T> rotateGTX(const _xquat<T>& q, float angle, float x, float y, float z)
    {
        // Axis of rotation must be normalised
        float len = glm::length(glm::_xvec3<T>(x, y, z));
        if(glm::abs(len - 1.0f) > 0.001f)
        {
            float oneOverLen = 1 / len;
            x *= oneOverLen;
            y *= oneOverLen;
            z *= oneOverLen;
        }

        angle = glm::radians(angle);
        float fSin = glm::sin(angle * 0.5f);

        return glm::_xquat<T>(glm::cos(angle * 0.5f), x * fSin, y * fSin, z * fSin);
    }

    template <typename T> 
    inline _xvec3<T> rotateGTX(const _xquat<T>& q, const _xvec3<T>& v)
    {
        _xquat<T> Result = q * v * inverseGTX(q);
        return _xvec3<T>(Result.x, Result.y, Result.z);
    }

    template <typename T> 
    inline _xvec4<T> rotateGTX(const _xquat<T>& q, const _xvec4<T>& v)
    {
        _xquat<T> Result = q * v * inverseGTX(q);
        return _xvec4<T>(v.w, Result.x, Result.y, Result.z);
    }

    template <typename T> 
    inline float angleGTX(const _xquat<T>& x)
    {
        return glm::cos(x.w) * 2.0f;
    }

    template <typename T> 
    inline _xvec3<T> axisGTX(const _xquat<T>& x)
    {
        T tmp1 = 1 - x.w * x.w;
        if(tmp1 <= 0.0f)
            return glm::_xvec3<T>(0, 0, 1);
        T tmp2 = 1 / glm::sqrt(tmp1);
        return glm::_xvec3<T>(x.x * tmp2, x.y * tmp2, x.z * tmp2);
    }

    template <typename T> 
    inline _xmat3<T> mat3GTX(const _xquat<T>& q)
    {
        _xmat3<T> Result(1);
        Result[0][0] = 1 - 2 * q.y * q.y - 2 * q.z * q.z;
        Result[0][1] = 2 * q.x * q.y + 2 * q.w * q.z;
        Result[0][2] = 2 * q.x * q.z - 2 * q.w * q.y;

        Result[1][0] = 2 * q.x * q.y - 2 * q.w * q.z;
        Result[1][1] = 1 - 2 * q.x * q.x - 2 * q.z * q.z;
        Result[1][2] = 2 * q.y * q.z + 2 * q.w * q.x;

        Result[2][0] = 2 * q.x * q.z + 2 * q.w * q.y;
        Result[2][1] = 2 * q.y * q.z - 2 * q.w * q.x;
        Result[2][2] = 1 - 2 * q.x * q.x - 2 * q.y * q.y;
        return Result;
    }

    template <typename T> 
    inline _xmat4<T> mat4GTX(const _xquat<T>& q)
    {
        _xmat4<T> Result = _xmat4<T>(1);
        Result[0][0] = 1 - 2 * q.y * q.y - 2 * q.z * q.z;
        Result[0][1] = 2 * q.x * q.y + 2 * q.w * q.z;
        Result[0][2] = 2 * q.x * q.z - 2 * q.w * q.y;

        Result[1][0] = 2 * q.x * q.y - 2 * q.w * q.z;
        Result[1][1] = 1 - 2 * q.x * q.x - 2 * q.z * q.z;
        Result[1][2] = 2 * q.y * q.z + 2 * q.w * q.x;

        Result[2][0] = 2 * q.x * q.z + 2 * q.w * q.y;
        Result[2][1] = 2 * q.y * q.z - 2 * q.w * q.x;
        Result[2][2] = 1 - 2 * q.x * q.x - 2 * q.y * q.y;
        return Result;
    }

    template <typename T> 
    inline _xquat<T> quatGTX(const _xmat3<T>& m)
    {
        T fourXSquaredMinus1 = m[0][0] - m[1][1] - m[2][2];
        T fourYSquaredMinus1 = m[1][1] - m[0][0] - m[2][2];
        T fourZSquaredMinus1 = m[2][2] - m[0][0] - m[1][1];
        T fourWSquaredMinus1 = m[0][0] + m[1][1] + m[2][2];
        
        int biggestIndex = 0;
        T fourBiggestSquaredMinus1 = fourWSquaredMinus1;
        if(fourXSquaredMinus1 > fourBiggestSquaredMinus1)
        {
            fourBiggestSquaredMinus1 = fourXSquaredMinus1;
            biggestIndex = 1;
        }
        if(fourYSquaredMinus1 > fourBiggestSquaredMinus1)
        {
            fourBiggestSquaredMinus1 = fourYSquaredMinus1;
            biggestIndex = 2;
        }
        if(fourZSquaredMinus1 > fourBiggestSquaredMinus1)
        {
            fourBiggestSquaredMinus1 = fourZSquaredMinus1;
            biggestIndex = 3;
        }

        T biggestVal = glm::sqrt(fourBiggestSquaredMinus1 + 1) * 0.5f;
        T mult = 0.25f / biggestVal;

        glm::_xquat<T> Result;
        switch(biggestIndex)
        {
        case 0:
            Result.w = biggestVal; 
            Result.x = (m[1][2] - m[2][1]) * mult;
            Result.y = (m[2][0] - m[0][2]) * mult;
            Result.z = (m[0][1] - m[1][0]) * mult;
            break;
        case 1:
            Result.w = (m[1][2] - m[2][1]) * mult;
            Result.x = biggestVal;
            Result.y = (m[0][1] + m[1][0]) * mult;
            Result.z = (m[2][1] + m[1][2]) * mult;
            break;
        case 2:
            Result.w = (m[2][0] - m[0][2]) * mult;
            Result.x = (m[0][1] + m[1][0]) * mult;
            Result.y = biggestVal;
            Result.z = (m[1][2] + m[2][1]) * mult;
            break;
        case 3:
            Result.w = (m[0][1] - m[1][0]) * mult;
            Result.x = (m[2][0] + m[0][2]) * mult;
            Result.y = (m[1][2] + m[2][1]) * mult;
            Result.z = biggestVal;
            break;
        }
        return Result;
    }

    template <typename T> 
    inline _xquat<T> quatGTX(const _xmat4<T>& m)
    {
        T fourWSquaredMinus1 = m[0][0] + m[1][1] + m[2][2];
        T fourXSquaredMinus1 = m[0][0] - m[1][1] - m[2][2];
        T fourYSquaredMinus1 = m[1][1] - m[0][0] - m[2][2];
        T fourZSquaredMinus1 = m[2][2] - m[0][0] - m[1][1];
        
        int biggestIndex = 0;
        T fourBiggestSquaredMinus1 = fourWSquaredMinus1;
        if(fourXSquaredMinus1 > fourBiggestSquaredMinus1)
        {
            fourBiggestSquaredMinus1 = fourXSquaredMinus1;
            biggestIndex = 1;
        }
        if(fourYSquaredMinus1 > fourBiggestSquaredMinus1)
        {
            fourBiggestSquaredMinus1 = fourYSquaredMinus1;
            biggestIndex = 2;
        }
        if(fourZSquaredMinus1 > fourBiggestSquaredMinus1)
        {
            fourBiggestSquaredMinus1 = fourZSquaredMinus1;
            biggestIndex = 3;
        }

        T biggestVal = glm::sqrt(fourBiggestSquaredMinus1 + 1) * 0.5f;
        T mult = 0.25f / biggestVal;

        glm::_xquat<T> Result;
        switch(biggestIndex)
        {
        case 0:
            Result.w = biggestVal;
            Result.x = (m[1][2] - m[2][1]) * mult;
            Result.y = (m[2][0] - m[0][2]) * mult;
            Result.z = (m[0][1] - m[1][0]) * mult;
            break;
        case 1:
            Result.w = (m[1][2] - m[2][1]) * mult;
            Result.x = biggestVal;
            Result.y = (m[0][1] + m[1][0]) * mult;
            Result.z = (m[2][1] + m[1][2]) * mult;
            break;
        case 2:
            Result.w = (m[2][0] - m[0][2]) * mult;
            Result.x = (m[0][1] + m[1][0]) * mult;
            Result.y = biggestVal;
            Result.z = (m[1][2] + m[2][1]) * mult;
            break;
        case 3:
            Result.w = (m[0][1] - m[1][0]) * mult;
            Result.x = (m[2][0] + m[0][2]) * mult;
            Result.y = (m[1][2] + m[2][1]) * mult;
            Result.z = biggestVal;
            break;
        }
        return Result;
    }

    template <typename T> 
    inline _xquat<T> angleAxisGTX(float angle, float x, float y, float z)
    {
        _xquat<T> result;
        // Axis of rotation must be normalised
        float len = glm::length(glm::_xvec3<T>(x, y, z));
        if(glm::abs(len - 1.0f) > 0.001f)
        {
            float oneOverLen = 1 / len;
            x *= oneOverLen;
            y *= oneOverLen;
            z *= oneOverLen;
        }

        angle = glm::radians(angle);
        float fSin = glm::sin(angle * 0.5f);

        result.w = glm::cos(angle * 0.5f);
        result.x = x * fSin;
        result.y = y * fSin;
        result.z = z * fSin;
        return result;
    }

    template <typename T> 
    inline _xquat<T> angleAxisGTX(float angle, const _xvec3<T>& v)
    {
        glm::_xquat<T> result;
        glm::_xvec3<T> v_normalized = glm::normalize(v);

        angle = glm::radians(angle);
        float fSin = glm::sin(angle * 0.5f);

        result.w = glm::cos(angle * 0.5f);
        result.x = v.x * fSin;
        result.y = v.y * fSin;
        result.z = v.z * fSin;
        return result;
    }

    template <typename T> T extractRealComponentGTX(const _xquat<T>& q)
    {
        T w = T(1.0) - q.x * q.x - q.y * q.y - q.z * q.z;
        if(w < T(0.0))
            return T(0.0);
        else
            return -glm::sqrt(w);
    }
}

#endif//__gtx_quaternion_inl__
