//////////////////////////////////////////////////////////////////////////////////
// OpenGL Mathematics Copyright (c) 2005 G-Truc Creation (www.g-truc.net)
//////////////////////////////////////////////////////////////////////////////////
// Author  : Christophe [Groove] Riccio (contact [at] g-truc [point] net)
// Created : 2006-01-16
// Updated : 2006-01-16
// Licence : This source is under GNU LGPL licence
// File    : gtx_random.inl
//////////////////////////////////////////////////////////////////////////////////

#ifndef __gtx_random_inl__
#define __gtx_random_inl__

#include "gtx_random.h"
#include <ctime>

namespace glm
{
    inline float rand1GTX()
    {
        return 2.0f * float(std::rand() - (RAND_MAX >> 1)) / float(RAND_MAX);
    }

    inline float rand1GTX(float Min, float Max)
    {
        return (glm::rand1GTX() * (Max - Min)) + Min;
    }
    
    inline _xvec2<float> rand2GTX()
    {
        float a = glm::rand1GTX(0.0f, 6.283185307179586476925286766559f);
        return _xvec2<float>(glm::cos(a), glm::sin(a));
    }

    inline _xvec2<float> rand2GTX(float Min, float Max)
    {
        return glm::rand2GTX() * rand1GTX(Min, Max);
    }

    inline _xvec3<float> rand3GTX()
    {
	    float z = glm::rand1GTX(-1.0f, 1.0f);
	    float a = glm::rand1GTX(0.0f, 3.1415926535897932384626433832795f);

        float r = glm::sqrt(1.0f - z * z);

	    float x = r * glm::cos(a);
	    float y = r * glm::sin(a);

        return _xvec3<float>(x, y, z);
    }

    inline _xvec3<float> rand3GTX(float Min, float Max)
    {
        return glm::rand3GTX() * rand1GTX(Min, Max);
    }

    inline _xvec4<float> rand4GTX()
    {
        return _xvec4<float>(rand3GTX(), 0.0f);
    }

    inline _xvec4<float> rand4GTX(float Min, float Max)
    {
        return glm::rand4GTX() * rand1GTX(Min, Max);
    }
}

#endif//__gtx_random_inl__
